<?php

/*
 +--------------------------------------------------------------------+
 | CiviCRM version 1.8                                                |
 +--------------------------------------------------------------------+
 | Copyright CiviCRM LLC (c) 2004-2007                                |
 +--------------------------------------------------------------------+
 | This file is a part of CiviCRM.                                    |
 |                                                                    |
 | CiviCRM is free software; you can copy, modify, and distribute it  |
 | under the terms of the Affero General Public License Version 1,    |
 | March 2002.                                                        |
 |                                                                    |
 | CiviCRM is distributed in the hope that it will be useful, but     |
 | WITHOUT ANY WARRANTY; without even the implied warranty of         |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.               |
 | See the Affero General Public License for more details.            |
 |                                                                    |
 | You should have received a copy of the Affero General Public       |
 | License along with this program; if not, contact CiviCRM LLC       |
 | at info[AT]civicrm[DOT]org.  If you have questions about the       |
 | Affero General Public License or the licensing  of CiviCRM,        |
 | see the CiviCRM license FAQ at http://civicrm.org/licensing        |
 +--------------------------------------------------------------------+
*/

/**
 *
 * @package CRM
 * @copyright CiviCRM LLC (c) 2004-2007
 * $Id$
 *
 */

$GLOBALS['_CRM_ACL_BAO_CACHE']['_cache'] =  null;

require_once 'CRM/ACL/DAO/Cache.php';

/**
 *  Access Control Cache
 */
class CRM_ACL_BAO_Cache extends CRM_ACL_DAO_Cache {

    

     function &build( $id ) {
        // in testing phase reset cache every time
        CRM_ACL_BAO_Cache::resetCache( );

        if ( ! $GLOBALS['_CRM_ACL_BAO_CACHE']['_cache'] ) {
            $GLOBALS['_CRM_ACL_BAO_CACHE']['_cache'] = array( );
        }
        
        if ( array_key_exists( $id, $GLOBALS['_CRM_ACL_BAO_CACHE']['_cache'] ) ) {
            return $GLOBALS['_CRM_ACL_BAO_CACHE']['_cache'][$id];
        }

        // check if this entry exists in db
        // if so retrieve and return
        $GLOBALS['_CRM_ACL_BAO_CACHE']['_cache'][$id] = CRM_ACL_BAO_Cache::retrieve( $id );
        if ( $GLOBALS['_CRM_ACL_BAO_CACHE']['_cache'][$id] ) {
            return $GLOBALS['_CRM_ACL_BAO_CACHE']['_cache'][$id];
        }

        require_once 'CRM/ACL/BAO/ACL.php';
        $GLOBALS['_CRM_ACL_BAO_CACHE']['_cache'][$id] = CRM_ACL_BAO_ACL::getAllByContact( $id );
        CRM_ACL_BAO_Cache::store( $id, $GLOBALS['_CRM_ACL_BAO_CACHE']['_cache'][$id] );
        return $GLOBALS['_CRM_ACL_BAO_CACHE']['_cache'][$id];
    }

     function retrieve( $id ) {
        $query = "
SELECT acl_id
  FROM civicrm_acl_cache
 WHERE contact_id = %1
";
        $params = array( 1 => array( $id, 'Integer' ) );
        $dao =& CRM_Core_DAO::executeQuery( $query, $params );

        $cache = array( );
        while ( $dao->fetch( ) ) {
            $cache[$dao->acl_id] = 1;
        }
        return $cache;
    }

     function store( $id, &$cache ) {
        foreach ( $cache as $aclID => $data ) {
            $dao =& new CRM_ACL_DAO_Cache( );
            if ( $id ) {
                $dao->contact_id = $id;
            }
            $dao->acl_id     = $aclID;

            $cache[$aclID] = 1;

            $dao->save( );
        }
    }

     function deleteEntry( $id ) {
        if ( $GLOBALS['_CRM_ACL_BAO_CACHE']['_cache'] &&
             array_key_exists( $id, $GLOBALS['_CRM_ACL_BAO_CACHE']['_cache'] ) ) {
            unset( $GLOBALS['_CRM_ACL_BAO_CACHE']['_cache'][$id] );
        }

        $query = "
DELETE FROM civicrm_acl_cache
WHERE contact_id = %1
";
        $params = array( 1 => array( $id, 'Integer' ) );
        $dao =& CRM_Core_DAO::executeQuery( $query, $params );
    }

     function updateEntry( $id ) {
        CRM_ACL_BAO_Cache::delete( $id );
        CRM_ACL_BAO_Cache::build( $id );
    }

    // deletes all the cache entries for this domain
     function resetCache( ) {
        $GLOBALS['_CRM_ACL_BAO_CACHE']['_cache'] = null;

        $domainID = CRM_Core_Config::domainID( );
        $query = "
DELETE     c 
FROM       civicrm_acl_cache c
INNER JOIN civicrm_acl       a ON c.acl_id = a.id
WHERE      a.domain_id = $domainID
";

        $dao =& CRM_Core_DAO::executeQuery( $query, $GLOBALS['_CRM_CORE_DAO']['_nullArray'] );
    }

}

?>